# eventing-backend

## Installation

```bash
$ npm install
```

### Edit environment variables

- `.env` standard environment
- `.env.test` testing environment, overrides .env file during testing

**Example .env:**
```bash
TYPEORM_CONNECTION=mysql
TYPEORM_HOST=localhost
TYPEORM_USERNAME=root
TYPEORM_PASSWORD=password
TYPEORM_DATABASE=eventing
TYPEORM_PORT=3306
TYPEORM_SYNCHRONIZE=false
TYPEORM_LOGGING=false
TYPEORM_ENTITIES=src/**/*.entity.ts
TYPEORM_MIGRATIONS=src/migration/**/*.ts
TYPEORM_SUBSCRIBERS=src/subscriber/**/*.ts
TYPEORM_ENTITIES_DIR=src
TYPEORM_MIGRATIONS_DIR=src/migration
TYPEORM_SUBSCRIBERS_DIR=src/subscriber
NODE_ENV=development
HOME_URL=eventing.dev

;Only use if deploying to Microsoft Azure:
MicrosoftAppId=
MicrosoftAppPassword=
```

**Available variables:**

- TypeORM: https://github.com/typeorm/typeorm/blob/master/docs/using-ormconfig.md#using-environment-variables

## Running the app

```bash
# Start mariaDB
$ npm run db:start

# Keep the DB structure up to date
$ npm run orm -- migration:run

# Insert the mock events
$ npm run orm -- query "$(cat mockData.sql)"

# development
$ npm run start

# watch mode
$ npm run start:dev

# production mode
$ npm run start:prod
```

## Test

```bash
# Start mariaDB for testing
$ npm run db:test

# e2e tests
$ npm run test:e2e

# unit tests
$ npm run test

# test coverage
$ npm run test:cov
```

**Beware, during testing, the DB structure gets updated automatically, bypassing the migration system. This however leads to broken migrations after that point. Running `npm run orm -- schema:drop` is a convenient way to reset the DB - and its DATA!!**

## Chatbot

This bot has been created using [Bot Framework](https://dev.botframework.com)

## Testing the bot using Bot Framework Emulator

[Bot Framework Emulator](https://github.com/microsoft/botframework-emulator) is a desktop application that allows bot developers to test and debug their bots on localhost or running remotely through a tunnel.

- Install the Bot Framework Emulator version 4.3.0 or greater from [here](https://github.com/Microsoft/BotFramework-Emulator/releases)

### Connect to the bot using Bot Framework Emulator

- Launch Bot Framework Emulator
- File -> Open Bot
- Enter a Bot URL of `http://localhost:3000/bot/messages`

## Deploy the bot to Azure

### Publishing Changes to Azure Bot Service

    ```bash
    # build the TypeScript bot before you publish
    npm run build
    ```

To learn more about deploying a bot to Azure, see [Deploy your bot to Azure](https://aka.ms/azuredeployment) for a complete list of deployment instructions.

## Further reading

- [Bot Framework Documentation](https://docs.botframework.com)
- [Bot Basics](https://docs.microsoft.com/azure/bot-service/bot-builder-basics?view=azure-bot-service-4.0)
- [Dialogs](https://docs.microsoft.com/en-us/azure/bot-service/bot-builder-concept-dialog?view=azure-bot-service-4.0)
- [Gathering Input Using Prompts](https://docs.microsoft.com/en-us/azure/bot-service/bot-builder-prompts?view=azure-bot-service-4.0)
- [Activity processing](https://docs.microsoft.com/en-us/azure/bot-service/bot-builder-concept-activity-processing?view=azure-bot-service-4.0)
- [Azure Bot Service Introduction](https://docs.microsoft.com/azure/bot-service/bot-service-overview-introduction?view=azure-bot-service-4.0)
- [Azure Bot Service Documentation](https://docs.microsoft.com/azure/bot-service/?view=azure-bot-service-4.0)
- [Azure CLI](https://docs.microsoft.com/cli/azure/?view=azure-cli-latest)
- [Azure Portal](https://portal.azure.com)
- [Language Understanding using LUIS](https://docs.microsoft.com/en-us/azure/cognitive-services/luis/)
- [Channels and Bot Connector Service](https://docs.microsoft.com/en-us/azure/bot-service/bot-concepts?view=azure-bot-service-4.0)
- [TypeScript](https://www.typescriptlang.org)
- [Restify](https://www.npmjs.com/package/restify)
- [dotenv](https://www.npmjs.com/package/dotenv)
