import { Injectable } from '@nestjs/common';
import { InjectRepository } from '@nestjs/typeorm';
import { FindConditions, Repository } from 'typeorm';

import { Broadcast } from './broadcast.entity';

@Injectable()
export class BroadcastService {

    constructor(
        @InjectRepository(Broadcast)
        private readonly repository: Repository<Broadcast>) {
    }

    public async one(conditions: FindConditions<Broadcast>): Promise<Broadcast> {
        return this.repository.findOneOrFail(conditions);
    }

    public async find(conditions: FindConditions<Broadcast>): Promise<Broadcast[]> {
        return this.repository.find(conditions);
    }

    public async create(title: string, body: string, replaySend?: boolean) {
        return await this.repository.save({
            title,
            body,
            replaySend
        });
    }

    public async update(id: string, title: string, body: string, replaySend?: boolean): Promise<Broadcast> {
        const broadcast = await this.one({ id });
        const updateObj: Partial<Broadcast> = {
            title,
            body,
            replaySend
        };
        const update = await this.repository.update(broadcast.id, updateObj);
        return Object.assign(broadcast, updateObj);
    }

    public async remove(broadcasts: Broadcast[]): Promise<void> {
        await this.repository.remove(broadcasts);
    }
}
