import { Event } from '../events/event.entity';

import { VEvent } from './VEvent';

export class VCalendar {

  public headers: any[] = [];

  public events: VEvent[] = [];

  public footer: any[] = [];

  constructor(calName, calScale, version, method, prodId) {
    this.createHeaders(calScale, calName, version, method, prodId);
    this.createFooter();
  }

  private createHeaders(calScale: string, calName: string, version: string, method: string, prodId: string) {
    this.headers = [
      {
        key: 'BEGIN',
        value: 'VCALENDAR'
      },
      {
        key: 'CALSCALE',
        value: calScale || 'GREGORIAN'
      },
      {
        key: 'VERSION',
        value: version || '2.0'
      },
      {
        key: 'X-WR-CALNAME',
        value: calName
      },
      {
        key: 'METHOD',
        value: method || 'PUBLISH'
      },
      {
        key: 'PRODID',
        value: prodId
      }
    ];
  }

  private createFooter() {
    this.footer = [
      {
        key: 'END',
        value: 'VCALENDAR'
      }
    ];
  }

  public addEvent(id: string, startDate: number, endDate: number, isAllDay: boolean, name: string, description: string, location: string, visibility: string, category: string, organizer: string, createdDate: number, lastModifiedDate: number) {
    const event = new VEvent(id, startDate, endDate, isAllDay, name, description, location, visibility, category, organizer, createdDate, lastModifiedDate);

    this.events.push(event);
  }

  private toICSLine(key: string, value: any) {
    if (!key || key === '' || !value) {
      return null;
    }

    value = typeof value === 'string' ? value : value.toString();

    key = key.replace(/[\\;,\n]/g, (match) => {
      return (match === '\n' ? '\\n' : ('\\' + match));
    });
    value = value.replace(/[\\;,\n]/g, (match) => {
      return (match === '\n' ? '\\n' : ('\\' + match));
    });
    return key + ':' + value;
  }

  public toICSString() {
    const lines = [];
    let line = '';

    this.headers.forEach((header) => {
      line = this.toICSLine(header.key, header.value);
      if (line) {
        lines.push(line);
      }
    });

    this.events.forEach((vEvent) => {
      vEvent.headers.forEach((header) => {
        line = this.toICSLine(header.key, header.value);
        if (line) {
          lines.push(line);
        }
      });

      vEvent.props.forEach((prop) => {
        line = this.toICSLine(prop.key, prop.value);
        if (line) {
          lines.push(line);
        }
      });

      vEvent.footer.forEach((item) => {
        line = this.toICSLine(item.key, item.value);
        if (line) {
          lines.push(line);
        }
      });
    });

    this.footer.forEach((item) => {
      line = this.toICSLine(item.key, item.value);
      if (line) {
        lines.push(line);
      }
    });

    return lines.join('\r\n');
  }

  public createCalendarFromEvents(events: Event[]) {
    /**
     * @TODO visibility, isAllDAY, type, organizer, modifiedDate
     */
    events.forEach((event) => {
      this.addEvent(event.id, event.dateStart, event.dateEnd, false, event.title, event.description, event.location, 'PUBLIC', 'Event', 'EVENTING', event.dateCreated, event.dateCreated);
    });
  }
}
