import { DateFormater } from '../utility/DateFormater';

export class VEvent {
  public headers: any[] = [];
  public props: any[] = [];
  public alarms: any[] = [];
  public footer: any[] = [];

  constructor(
    id: string, startDate: number, endDate: number, isAllDay: boolean,
    name: string, description: string, location: string, visibility: string, category: string,
    organizer: string, createdDate: number, lastModifiedDate: number) {
    this.buildHeaders();
    this.buildProps(id, startDate, endDate, isAllDay, name, description, location, visibility, category, createdDate, lastModifiedDate, organizer);
    this.buildFooter();
  }

  private buildHeaders() {
    this.headers = [
      {
        key: 'BEGIN',
        value: 'VEVENT'
      }
    ];
  }

  private buildProps(id: string, startDate: number, endDate: number, isAllDay: boolean, name: string, description: string, location: string, visibility: string, category: string, createdDate: number, lastModifiedDate: number, organizer: string) {
    this.props = [
      {
        key: 'TRANSP',
        value: 'TRANSPARENT'
      },
      {
        key: 'UID',
        value: id
      },
      {
        key: 'DTSTAMP',
        value: DateFormater.toICSUTC(startDate, false)
      },
      {
        key: 'DTSTART' + (isAllDay ? ';VALUE=DATE' : ''),
        value: DateFormater.toICSUTC(startDate, isAllDay)
      },
      {
        key: 'DTEND' + (isAllDay ? ';VALUE=DATE' : ''),
        value: DateFormater.toICSUTC(endDate, isAllDay)
      },
      {
        key: 'SUMMARY',
        value: name
      },
      {
        key: 'DESCRIPTION',
        value: description
      },
      {
        key: 'LOCATION',
        value: location
      },
      {
        key: 'CLASS',
        value: visibility || 'PUBLIC'
      },
      {
        key: 'CATEGORIES',
        value: category
      },
      {
        key: 'STATUS',
        value: 'CONFIRMED'
      },
      {
        key: 'SEQUENCE',
        value: 0
      },
      {
        key: 'CREATED',
        value: DateFormater.toICSUTC(createdDate, false)
      },
      {
        key: 'LAST-MODIFIED',
        value: DateFormater.toICSUTC(lastModifiedDate, false)
      },
      /**
       * @TODO organizer with mailto
       * https://www.kanzaki.com/docs/ical/organizer.html
       */
      {
        key: 'ORGANIZER;CN=' + organizer,
        value: organizer
      }
    ];
  }

  private buildFooter() {
    this.footer = [
      {
        key: 'END',
        value: 'VEVENT'
      }
    ];
  }
}
