import { DynamicModule, Module } from '@nestjs/common';
import { ConfigModule, ConfigService } from '@nestjs/config';
import { BotFrameworkAdapter, ConversationState, InputHints, MemoryStorage, UserState } from 'botbuilder';

// See https://aka.ms/about-bot-adapter to learn more about adapters.
@Module({})
export class ChatbotAdapterModule {

    public static register(): DynamicModule {
        // TODO: use persistent storage provider
        const storage = new MemoryStorage();

        return {
            module: ChatbotAdapterModule,
            imports: [ConfigModule],
            exports: [
                BotFrameworkAdapter,
                ConversationState,
                UserState
            ],
            providers: [
                {
                    provide: ConversationState,
                    useValue: new ConversationState(storage)
                },
                {
                    provide: UserState,
                    useValue: new UserState(storage)
                },
                {
                    provide: BotFrameworkAdapter,
                    useFactory: (configService: ConfigService) => {
                        const adapter = new BotFrameworkAdapter({
                            appId: configService.get('MicrosoftAppId'),
                            appPassword: configService.get('MicrosoftAppPassword')
                        });

                        adapter.onTurnError = async (context, error) => {
                            // This check writes out errors to console log .vs. app insights.
                            // NOTE: In production environment, you should consider logging this to Azure
                            //       application insights.
                            console.error(`\n [onTurnError]: ${ error } \n`, error.stack);

                            if (configService.get('NODE_ENV') === 'development') {
                                // Send a trace activity, which will be displayed in Bot Framework Emulator
                                await context.sendTraceActivity(
                                    'OnTurnError Trace',
                                    `${ error }`,
                                    'https://www.botframework.com/schemas/error',
                                    'TurnError'
                                    );
                            } else {
                                // Send a message to the user
                                const onTurnErrorMessage = `Sorry, it looks like something went wrong!`;
                                await context.sendActivity(onTurnErrorMessage, onTurnErrorMessage, InputHints.ExpectingInput);
                            }
                        };
                        return adapter;
                    },
                    inject: [ConfigService]
                }
            ]
        };
    }
}
