import { ArgumentsHost, Catch, ExceptionFilter, HttpException, HttpStatus } from '@nestjs/common';

/**
 * Used to globally catch exceptions,
 * restructure them and send them to the user
 * as well as output to the server console.
 */
@Catch()
export class HttpExceptionFilter implements ExceptionFilter {
  public catch(exception: Error, host: ArgumentsHost) {
    const ctx = host.switchToHttp();
    const response = ctx.getResponse();
    const request = ctx.getRequest();

    // Set status 500 for non HTTP exceptions
    const status =
      exception instanceof HttpException
        ? exception.getStatus()
        : HttpStatus.INTERNAL_SERVER_ERROR;

    const error = {
        statusCode: status,
        timestamp: new Date().toISOString(),
        msg: exception.message,
        path: request.url
    };

    console.error({ error, stack: exception.stack });

    response.status(status).json(error);
  }
}
