import { Body, Controller, Delete, Get, Param, Post, Query, UseGuards, UseInterceptors } from '@nestjs/common';

import { Session } from '../user/session.entity';
import { UserSession } from '../user/user.decorator';
import { UserSessionGuard } from '../user/user.guard';

import { OrganiserService } from './organiser.service';

@Controller('/api/organiser')
export class OrganiserController {

    constructor(
        private organiserService: OrganiserService) {}

    @Get()
    @UseGuards(UserSessionGuard)
    public async mine(@UserSession() session: Session) {
        const orga = await this.organiserService.getManagedOrganizers(session.user.id);
        return orga;
    }

    @Get(':uuid')
    public async one(@Param('uuid') uuid: string) {
        // Todo: use this route to send the subscriber count
        return await this.organiserService.one({ id: uuid });
    }

    @Post()
    @UseGuards(UserSessionGuard)
    public async create(@Body('name') name: string, @UserSession() session: Session) {
        return await this.organiserService.create(name, session.user.id);
    }

    @Delete(':uuid')
    @UseGuards(UserSessionGuard)
    public async remove(@Param('uuid') uuid: string, @UserSession() session: Session) {
        const organiser = await this.organiserService.one({ id: uuid });
        return await this.organiserService.remove(organiser, session.user.id);
    }

    @Post(':uuid/admin')
    @UseGuards(UserSessionGuard)
    public async addAdmin(@Param('uuid') uuid: string, @Body('userId') userId: string, @UserSession() session: Session) {
        return await this.organiserService.addAdmin(uuid, userId, session.user.id);
    }

    @Delete(':uuid/admin')
    @UseGuards(UserSessionGuard)
    public async removeAdmin(@Param('uuid') uuid: string, @Body('userId') userId: string, @UserSession() session: Session) {
        await this.organiserService.removeAdmin(uuid, userId, session.user.id);
    }

    @Post(':uuid/subscribe')
    @UseGuards(UserSessionGuard)
    public async subscribe(@Param('uuid') uuid: string, @UserSession() session: Session) {
        await this.organiserService.subscribe(uuid, session.user.id);
    }

    @Delete(':uuid/subscribe')
    @UseGuards(UserSessionGuard)
    public async unsubscribe(@Param('uuid') uuid: string, @UserSession() session: Session) {
        await this.organiserService.unsubscribe(uuid, session.user.id);
    }

    @Get(':uuid/subscribers')
    @UseGuards(UserSessionGuard)
    public async subscribers(@Param('uuid') uuid: string, @UserSession() session: Session, @Query('offset') skip?: 0) {
        await this.organiserService.getAdmin(uuid, session.user.id); // crashes if the user doesn't have the rights
        return await this.organiserService.getSubscribers(uuid, skip, 25);
    }
}
