import { forwardRef, Module } from '@nestjs/common';
import { TypeOrmModule } from '@nestjs/typeorm';

import { CalendarsModule } from '../calendars/calendars.module';
import { EventsModule } from '../events/events.module';
import { EventsService } from '../events/events.service';
import { JobsModule } from '../jobs/jobs.module';
import { OrganiserModule } from '../organiser/organiser.module';

import { SessionController } from './session.controller';
import { Session } from './session.entity';
import { SessionService } from './session.service';
import { UserController } from './user.controller';
import { User } from './user.entity';
import { UserSessionGuard } from './user.guard';
import { UserService } from './user.service';
import { UserMeta } from './usermeta.entity';
import { UserMetaService } from './usermeta.service';

@Module({
    imports: [
        TypeOrmModule.forFeature([User, Session, UserMeta]),
        JobsModule,
        forwardRef(() => OrganiserModule),
        forwardRef(() => EventsModule),
        forwardRef(() => CalendarsModule)
    ],
    controllers: [
        UserController,
        SessionController
    ],
    providers: [
        UserService,
        UserMetaService,
        SessionService,
        EventsService,
        UserSessionGuard
    ],
    exports: [
        TypeOrmModule,
        UserService,
        UserMetaService,
        SessionService,
        UserSessionGuard
    ]
})
export class UserModule { }
