import { Test } from '@nestjs/testing';
import * as supertest from 'supertest';
import { Connection } from 'typeorm';

import { AppModule } from './../src/app.module';
import { Globals } from './common';
import { organiserTests, organiserTestsDestroy } from './organiser.e2e-defs';
import { userTests, userTestsDestroy } from './user.e2e-defs';

const testGlobals: Globals = {};

beforeAll(async () => {
  const moduleFixture = await Test.createTestingModule({
    imports: [AppModule]
  }).compile();

  testGlobals.app = moduleFixture.createNestApplication();
  await testGlobals.app.init();

  testGlobals.req = supertest(testGlobals.app.getHttpServer());

  const conn = testGlobals.app.get(Connection);
  await conn.synchronize(true);
});

afterAll(async () => {
  await testGlobals.app.close();
});

describe('Is initialized', () => {

  it('Should be connected to the DB', async () => {
    const conn = testGlobals.app.get(Connection);
    const res = await conn.query('SHOW TABLES;');
    expect(conn.isConnected).toBeTruthy();
    expect(res.length).toEqual(conn.entityMetadatas.length);
  });

});

describe('Create user', userTests.bind(testGlobals));

describe('Create organiser', organiserTests.bind(testGlobals));

describe('Delete organiser', organiserTestsDestroy.bind(testGlobals));

describe('Delete user', userTestsDestroy.bind(testGlobals));
