import * as faker from 'faker';

import { Organiser } from '../src/organiser/organiser.entity';
import { OrganiserService } from '../src/organiser/organiser.service';

import { Globals } from './common';

export function organiserTests() {
    const testGlobals: Globals = this;
    let testOrganiser: Organiser;
    let organiserService: OrganiserService;

    beforeAll(async () => {
        organiserService = testGlobals.app.get(OrganiserService);
    });

    it('Create organiser', async () => {
        const name = faker.company.companyName();

        const { body } = await testGlobals.req
        .post('/api/organiser')
        .set('Cookie', testGlobals.cookies)
        .send({ name })
        .expect(201);

        expect(body).toEqual({
            name,
            id: expect.any(String),
            desc: null,
            dateCreated: expect.any(String)
        } as Organiser);
        testOrganiser = body;
    });

    it('Subscribe to organiser', async () => {
        const getSubCount = async () => (await organiserService.getSubscribers(testOrganiser.id, 0, 100)).length;
        const addSub = async () => {
            await testGlobals.req
            .post('/api/organiser/' + testOrganiser.id + '/subscribe')
            .set('Cookie', testGlobals.cookies)
            .expect(201);

            expect(await getSubCount()).toEqual(1);
        };

        await addSub();

        // remove it
        await testGlobals.req
        .delete('/api/organiser/' + testOrganiser.id + '/subscribe')
        .set('Cookie', testGlobals.cookies)
        .expect(200);
        expect(await getSubCount()).toEqual(0);

        // add it again
        await addSub();
        expect(await getSubCount()).toEqual(1);
    });
}

export function organiserTestsDestroy() {
    const testGlobals: Globals = this;
    let testOrganiserId: string;
    let organiserService: OrganiserService;

    beforeAll(async () => {
        const { body } = await testGlobals.req
        .get('/api/organiser')
        .set('Cookie', testGlobals.cookies)
        .expect(200);

        expect(body[1]).toEqual(1);

        testOrganiserId = body[0][0];
        organiserService = testGlobals.app.get(OrganiserService);
    });

    it('Delete an organiser', async () => {
        await testGlobals.req
        .delete('/api/organiser/' + testOrganiserId)
        .set('Cookie', testGlobals.cookies)
        .expect(200);

        const exists = await organiserService.exists({ id: testOrganiserId });
        expect(exists).toBeFalsy();
    });
}
